import numpy as np
import h5py
from .artist_bridge import CylindricalProjection
from .mesh_object import MeshObject
from pathlib import Path
import json



def to_dict(mesh_object = MeshObject) -> dict:
    return mesh_object.as_dict()


def record_atlas(cylindrical_projection: CylindricalProjection, mesh_list: list[MeshObject], 
                atlas_name: str, save_folder: Path,
                map_bounding_box: tuple[float]=(-50., 50.), number_of_maps: int = 4) -> Path:
    
    grid = np.linspace(map_bounding_box[0], map_bounding_box[1], number_of_maps)
    file_index = len(list(save_folder.glob('*.h5')))+1
    save_path = save_folder / f'{file_index:05}_{atlas_name}.h5'
    file = h5py.File(save_path, 'a')
    projection = file.require_dataset(
        'maps', 
        shape=(cylindrical_projection.x_px, cylindrical_projection.y_px, 0),  # Initial shape with third dimension as 0
        maxshape=(cylindrical_projection.x_px, cylindrical_projection.y_px, None), 
        dtype=np.float32)

    projection_points = file.require_dataset(
        'positions', 
        shape=(3, 0),  # Initial shape with third dimension as 0
        maxshape=(3, None), 
        dtype=np.float32)
    

    mesh_list_dict = list(map(to_dict, mesh_list))
    print(mesh_list_dict)
    
    file.attrs['mesh_list'] = json.dumps(mesh_list_dict)

    

    for x in grid:
        for y in grid:
            for z in grid:
                # x, y, z = 0, y, 0
                print(x, y, z)
                position = np.array([x, y, z])
                
                image = cylindrical_projection.compute_projection(position, output_full_ray_projection=True)
                current_size = projection.shape[2]
                new_size = current_size + 1
                projection.resize((cylindrical_projection.x_px, cylindrical_projection.y_px, new_size))
                projection[:, :, current_size:new_size] = image[:, :, np.newaxis].astype(np.float32)

                projection_points.resize((3, new_size))
                projection_points[:, current_size:new_size] = position.reshape((3, 1))

    return save_path
