from CyXTraX import CylindricalProjection, load_mesh, scones_mesh, MeshObject
import numpy as np
from matplotlib import pyplot as plt


def main():
    # Create the cylindrical projection interface
    cyl_proj = CylindricalProjection()


    # Load the test object 'olaf'. It is automatically installed to your site-packages
    print(f'STL Path: {scones_mesh.object_path}')

    # Obect are loaded in a dataclass: MeshObject.
    print(f'Olaf is a MeshObject: {isinstance(scones_mesh, MeshObject)}')
    scones_mesh.position_mm = np.array([0, 0, 0])
    scones_mesh.orientation_quat = np.array([0., -0., 0., 0.1])
    
    
    # The `load_mesh` function load a list of MeshObject
    load_mesh(cyl_proj, [scones_mesh])

    # Set local position of cylindrical projection
    map_position = np.array([0, 0, 0])

    # Generate local cylindrical projection
    half_rays = cyl_proj.compute_projection(map_position, output_full_ray_projection=False)
    # output_full_ray_projection=True is the same As 
    # half_rays = cyl_proj.compute_projection(position, output_full_ray_projection=False)
    # full_rays = cyl_proj.convert_rays(half_rays)
    full_rays = cyl_proj.compute_projection(map_position, output_full_ray_projection=True)
    

    # Make some nice plots!
    fig = plt.figure(figsize=(12, 6))

    ax1 = fig.add_subplot(121)
    ax1.imshow(half_rays)
    ax1.set_xlabel(r'$Z$ / mm')
    ax1.set_ylabel(r'$\alpha$ / rad')

    ax2 = fig.add_subplot(122)
    ax2.imshow(full_rays)
    ax2.set_xlabel(r'$Z$ / mm')
    ax2.set_ylabel(r'$\alpha$ / rad')

    plt.show()


if __name__ == '__main__':
    main()