from CyXTraX import CylindricalProjection, load_atlas, load_mesh
from CyXTraX.mapping import map_geometry_2_projection, map_source_2_cylinder
from pathlib import Path
from jax import numpy as jnp
from jax.scipy.spatial.transform import Rotation
from artistlib import SAVEMODES, utility
from matplotlib import pyplot as plt


# Just get the temp folder to store the atlas
FOLDER = Path(__file__)
TEMP_FOLDER = FOLDER.parent.parent / 'temp'


def main():
    files = list(TEMP_FOLDER.glob('*.h5'))
    if not files:
        raise FileNotFoundError('No .h5 files in temp folder. Run Example 02!')

    maps, points, mesh_object_list = load_atlas(files[-1])
    cyl_proj = CylindricalProjection()
    
    load_mesh(cyl_proj, mesh_object_list, False)
    print(f'Cone Beam Mode set: {cyl_proj.cone_mode}')

    rot = 30
    x, y = 2000 * jnp.cos(rot * jnp.pi / 180), 500 * jnp.sin(rot * jnp.pi / 180)
    source_position = jnp.array([x, y, 0.]).reshape((1, 3))
    detector_position = jnp.array([-x, -y, -10.]).reshape((1, 3))
    detctor_orientation_scipy = Rotation.from_euler('xyz', [0, -90, rot], degrees=True)

    temp_projection = Path('temp.tif')
    detctor_orientation = detctor_orientation_scipy.as_quat().reshape((1, -1))

    cyl_proj.api.translate('S', source_position[0, 0], source_position[0, 1], source_position[0, 2])
    cyl_proj.api.translate('D', detector_position[0, 0], detector_position[0, 1], detector_position[0, 2])
    cyl_proj.api.rotate_from_quat('D', detctor_orientation[0])
    cyl_proj.api.save_image(temp_projection, save_mode=SAVEMODES.FLOAT_TIFF)

    projection, geometry = utility.load_projection(temp_projection)

    mapped_values, uv_px = map_geometry_2_projection(
            source_position, 
            detector_position,
            detctor_orientation,
            points,
            projection, 
            cyl_proj.pixel_pitch_mm)
        
    values_calc, angles_calc = map_source_2_cylinder(
        source_position, 
        maps, 
        points, 
        cyl_proj.detector_radius_mm, 
        cyl_proj.x_px,
        cyl_proj.x_resolution_mm)
    
    for i in range(values_calc.shape[0]):
        print(values_calc[i], mapped_values[i])
        map_ = maps[:, :, i]
        value = map_[int(angles_calc[0, i]), int(angles_calc[1, i])]
        print(value)

    fig = plt.figure()
    ax1 = fig.add_subplot(111)
    ax1.imshow(projection)
    ax1.scatter(uv_px[0], uv_px[1])

    plt.show()

    

if __name__ == '__main__':
    main()